/*
 * Copyright (C) 2008 Michael Lamothe
 *
 * This file is part of Me TV
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Library General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor Boston, MA 02110-1301,  USA
 */

#ifndef __GTK_EPG_WIDGET__
#define __GTK_EPG_WIDGET__

#include <gtk/gtk.h>
#include "xml.hh"
#include "epg.hh"

class Timer
{
private:
	GTimer* timer;
public:
	Timer()
	{
		timer = g_timer_new();
	}

	~Timer()
	{
		g_timer_destroy (timer);
	}
	
	void reset()
	{
		g_timer_reset(timer);
	}
		
	gulong elapsed()
	{
		gulong microseconds;
		g_timer_elapsed (timer, &microseconds);
		return microseconds;
	}
};

class GtkEpgWidget
{
private:
	xmlNodePtr selected_event;
	gint offset;
	gsize span_hours;
	gsize span_minutes;
	gsize span_seconds;
	gsize last_number_rows;
	gsize last_number_columns;
	gsize number_rows;
	gsize number_columns;
	
	GtkWidget* vbox_epg;
	GtkWidget* table_epg;
	GtkWidget* scrolled_window_epg;
	GtkWidget* button_epg_previous;
	GtkWidget* button_epg_now;
	GtkWidget* button_epg_next;
	Timer timer;

	static void button_program_clicked(GtkButton* button, gpointer data);
	static void button_channel_name_clicked(GtkButton* button, gpointer data);
	static void button_epg_previous_clicked(GtkButton* button, gpointer data);
	static void button_epg_now_clicked(GtkButton* button, gpointer data);
	static void button_epg_next_clicked(GtkButton* button, gpointer data);

	String encode(const String& s);
	void set_offset(gint value);
	void update_table();
	GtkWidget* attach_button(const String& text, guint left_attach, guint right_attach, guint top_attach, guint bottom_attach);
	GtkWidget* attach_label(const String& text, guint left_attach, guint right_attach, guint top_attach, guint bottom_attach);
	void attach_widget(GtkWidget* widget, guint left_attach, guint right_attach, guint top_attach, guint bottom_attach);
	void create_channel_row(Channel& channel, guint row, gboolean selected, Epg& epg, guint now, guint start_time, guint end_time);

public:
	GtkEpgWidget();
	void update();
	void set_selected_event(xmlNodePtr event);
	void show();
	void hide();
	gboolean is_visible();
};

#endif
